==> gtk2-aurora-1.2.tar.bz2: gtk2-aurora-1.2/src/support.c <==
#include "support.h"

GtkTextDirection
aurora_get_direction (GtkWidget *widget)
{
	GtkTextDirection dir;
	
	if (widget)
		dir = gtk_widget_get_direction (widget);
	else
		dir = GTK_TEXT_DIR_LTR;
	
	return dir;
}


/* Widget Type Lookups/Macros
   
   Based on/modified from functions in
   Smooth-Engine.
*/ 
gboolean
aurora_object_is_a (const GObject * object, const gchar * type_name)
{
  gboolean result = FALSE;
 
  if ((object))
    {
      GType tmp = g_type_from_name (type_name);

      if (tmp)
	result = g_type_check_instance_is_a ((GTypeInstance *) object, tmp);
    }
 
  return result;
}

 
gboolean
aurora_is_combo_box_entry (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_COMBO_BOX_ENTRY (widget->parent))
	result = TRUE;
      else
	result = aurora_is_combo_box_entry (widget->parent);
    }
  return result;
}
 
static gboolean
aurora_combo_box_is_using_list (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if (AURORA_IS_COMBO_BOX (widget))
    {
      gboolean *tmp = NULL;
 
      gtk_widget_style_get (widget, "appears-as-list", &result, NULL);

      if (tmp)
	result = *tmp;
    }
 
  return result;
}
 
gboolean
aurora_is_combo_box (GtkWidget * widget, gboolean as_list)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_COMBO_BOX (widget->parent))
        {
          if (as_list)
            result = (aurora_combo_box_is_using_list(widget->parent));
          else
            result = (!aurora_combo_box_is_using_list(widget->parent));
        }
      else
	result = aurora_is_combo_box (widget->parent, as_list);
    }
  return result;
}
 
gboolean
aurora_is_combo (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_COMBO (widget->parent))
	result = TRUE;
      else
	result = aurora_is_combo (widget->parent);
    }
  return result;
}
 
gboolean
aurora_is_in_combo_box (GtkWidget * widget)
{
  return ((aurora_is_combo (widget) || aurora_is_combo_box (widget, TRUE) || aurora_is_combo_box_entry (widget)));
}
 
gboolean
aurora_is_tree_column_header (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_TREE_VIEW (widget->parent))
	result = TRUE;
      else
	result = aurora_is_tree_column_header (widget->parent);
    }
  return result;
}

gboolean
aurora_is_toolbar_item (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent)) {
    if ((AURORA_IS_BONOBO_TOOLBAR (widget->parent))
	|| (AURORA_IS_BONOBO_DOCK_ITEM (widget->parent))
	|| (AURORA_IS_EGG_TOOLBAR (widget->parent))
	|| (AURORA_IS_TOOLBAR (widget->parent))
	|| (AURORA_IS_HANDLE_BOX (widget->parent)))
      result = TRUE;
    else
      result = aurora_is_toolbar_item (widget->parent);
  }
  return result;
}
 
gboolean
aurora_is_panel_widget_item (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_PANEL_WIDGET (widget->parent))
	result = TRUE;
      else
	result = aurora_is_panel_widget_item (widget->parent);
    }
  return result;
}
 
gboolean 
aurora_is_bonobo_dock_item (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget))
    {
      if (AURORA_IS_BONOBO_DOCK_ITEM(widget) || AURORA_IS_BONOBO_DOCK_ITEM (widget->parent))
	result = TRUE;
      else if (AURORA_IS_BOX(widget) || AURORA_IS_BOX(widget->parent))
        {
          GtkContainer *box = AURORA_IS_BOX(widget)?GTK_CONTAINER(widget):GTK_CONTAINER(widget->parent);
          GList *children = NULL, *child = NULL;
 
          children = gtk_container_get_children(box);
              
          for (child = g_list_first(children); child; child = g_list_next(child))
            {
	      if (AURORA_IS_BONOBO_DOCK_ITEM_GRIP(child->data))
	        {
	          result = TRUE;
	          child = NULL;
	        }
            }	            
         
          if (children)   
  	    g_list_free(children);
	}
    }
  return result;
}

static GtkWidget *
aurora_find_combo_box_entry_widget (GtkWidget * widget)
{
  GtkWidget *result = NULL;

  if (widget)
    {
      if (AURORA_IS_COMBO_BOX_ENTRY (widget))
	result = widget;
      else
	result = aurora_find_combo_box_entry_widget (widget->parent);
    }

  return result;
}

static GtkWidget *
aurora_find_combo_box_widget (GtkWidget * widget, gboolean as_list)
{
  GtkWidget *result = NULL;
 
  if (widget)
    {
      if (AURORA_IS_COMBO_BOX (widget))
        {
          if (as_list)
            result = (aurora_combo_box_is_using_list(widget))?widget:NULL;
          else
            result = (!aurora_combo_box_is_using_list(widget))?widget:NULL;
        }
      else
	result = aurora_find_combo_box_widget (widget->parent, as_list);
    }
  return result;
}
 
static GtkWidget *
aurora_find_combo_widget (GtkWidget * widget)
{
  GtkWidget *result = NULL;
 
  if (widget)
    {
      if (AURORA_IS_COMBO (widget))
	result = widget;
      else
	result = aurora_find_combo_widget(widget->parent);
    }
  return result;
}

GtkWidget*
aurora_find_combo_box_widget_parent (GtkWidget * widget)
{
   GtkWidget *result = NULL;
   
   if (!result)
     result = aurora_find_combo_widget(widget);
  
   if (!result)
     result = aurora_find_combo_box_widget(widget, TRUE);

   if (!result)
     result = aurora_find_combo_box_entry_widget(widget);

  return result;
}


gboolean
aurora_widget_is_ltr (GtkWidget *widget)
{
	GtkTextDirection dir = GTK_TEXT_DIR_NONE;
	
	if (AURORA_IS_WIDGET (widget))
		dir = gtk_widget_get_direction (widget);

	if (dir == GTK_TEXT_DIR_NONE)
		dir = gtk_widget_get_default_direction ();

	if (dir == GTK_TEXT_DIR_RTL)
		return FALSE;
	else
		return TRUE;
}




/***********************************************
 * ge_hsb_from_color -
 *  
 *   Get HSB values from RGB values.
 *
 *   Modified from Smooth but originated in GTK+
 ***********************************************/
static void
aurora_hsb_from_color (const AuroraRGB *color, 
                        gdouble *hue, 
                        gdouble *saturation,
                        gdouble *brightness) 
{
	gdouble min, max, delta;
	gdouble red, green, blue;

	red = color->r;
	green = color->g;
	blue = color->b;
  
	if (red > green)
	{
		max = MAX(red, blue);
		min = MIN(green, blue);
	}
	else
	{
		max = MAX(green, blue);
		min = MIN(red, blue);
	}
  
	*brightness = (max + min) / 2;
 	
	if (fabs(max - min) < 0.0001)
	{
		*hue = 0;
		*saturation = 0;
	}	
	else
	{
		if (*brightness <= 0.5)
			*saturation = (max - min) / (max + min);
		else
			*saturation = (max - min) / (2 - max - min);
       
		delta = max -min;
 
		if (red == max)
			*hue = (green - blue) / delta;
		else if (green == max)
			*hue = 2 + (blue - red) / delta;
		else if (blue == max)
			*hue = 4 + (red - green) / delta;
 
		*hue *= 60;
		if (*hue < 0.0)
			*hue += 360;
	}
}
 
/***********************************************
 * ge_color_from_hsb -
 *  
 *   Get RGB values from HSB values.
 *
 *   Modified from Smooth but originated in GTK+
 ***********************************************/
#define MODULA(number, divisor) (((gint)number % divisor) + (number - (gint)number))
static void
aurora_color_from_hsb (gdouble hue, 
                        gdouble saturation,
                        gdouble brightness, 
                        AuroraRGB *color)
{
	gint i;
	gdouble hue_shift[3], color_shift[3];
	gdouble m1, m2, m3;

	if (!color) return;
  	  
	if (brightness <= 0.5)
		m2 = brightness * (1 + saturation);
	else
		m2 = brightness + saturation - brightness * saturation;
 
	m1 = 2 * brightness - m2;
 
	hue_shift[0] = hue + 120;
	hue_shift[1] = hue;
	hue_shift[2] = hue - 120;
 
	color_shift[0] = color_shift[1] = color_shift[2] = brightness;	
 
	i = (saturation == 0)?3:0;
 
	for (; i < 3; i++)
	{
		m3 = hue_shift[i];
 
		if (m3 > 360)
			m3 = MODULA(m3, 360);
		else if (m3 < 0)
			m3 = 360 - MODULA(ABS(m3), 360);
 
		if (m3 < 60)
			color_shift[i] = m1 + (m2 - m1) * m3 / 60;
		else if (m3 < 180)
			color_shift[i] = m2;
		else if (m3 < 240)
			color_shift[i] = m1 + (m2 - m1) * (240 - m3) / 60;
		else
			color_shift[i] = m1;
	}	
 
	color->r = color_shift[0];
	color->g = color_shift[1];
	color->b = color_shift[2];	
}

void
aurora_shade (const AuroraRGB *base, AuroraRGB *composite, double shade_ratio)
{
	gdouble hue = 0;
	gdouble saturation = 0;
	gdouble brightness = 0;
 
	g_return_if_fail (base && composite);

	aurora_hsb_from_color (base, &hue, &saturation, &brightness);
 
	brightness = MIN(brightness*shade_ratio, 1.0);
	brightness = MAX(brightness, 0.0);
  
	aurora_color_from_hsb (hue, saturation, brightness, composite);
}


void
aurora_match_lightness (const AuroraRGB * a, AuroraRGB * b)
{

  double min,max, l;
    
  if (a->r > a->g)
	{
		if (a->r > a->b)
			max = a->r;
		else
			max = a->b;
	
		if (a->g < a->b)
			min = a->g;
		else
			min = a->b;
	}
	else
	{
		if (a->g > a->b)
			max = a->g;
		else
			max = a->b;
	
		if (a->r < a->b)
			min = a->r;
		else
			min = a->b;
	}

	l = (max + min) / 2;
    
		
	gdouble hue = 0;
	gdouble saturation = 0;
	gdouble brightness = 0;
 
	aurora_hsb_from_color (b, &hue, &saturation, &brightness);
 
  brightness = l;
  
	aurora_color_from_hsb (hue, saturation, brightness, b);
		
}

gdouble
aurora_get_lightness (const AuroraRGB * a)
{

  double min,max;
    
  if (a->r > a->g)
	{
		if (a->r > a->b)
			max = a->r;
		else
			max = a->b;
	
		if (a->g < a->b)
			min = a->g;
		else
			min = a->b;
	}
	else
	{
		if (a->g > a->b)
			max = a->g;
		else
			max = a->b;
	
		if (a->r < a->b)
			min = a->r;
		else
			min = a->b;
	}

	return (max + min) / 2.0;
}


void
aurora_mix_color (const AuroraRGB * a, AuroraRGB * b, gdouble mix_factor, AuroraRGB * c)
{
	g_return_if_fail (a && b && c);

	c->r = a->r * (1-mix_factor) + b->r * mix_factor;
	c->g = a->g * (1-mix_factor) + b->g * mix_factor;
	c->b = a->b * (1-mix_factor) + b->b * mix_factor;
}


void aurora_gtk_treeview_get_header_index (GtkTreeView *tv, GtkWidget *header,
                                    gint *column_index, gint *columns,
                                    gboolean *resizable, gboolean *sorted)
{
	GList *list, *list_start;
	*column_index = *columns = 0;
	list_start = list = gtk_tree_view_get_columns (tv);

	do
	{
		GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN(list->data);
		if ( column->button == header )
		{
			*column_index = *columns;
			*resizable = column->resizable;
			*sorted = column->show_sort_indicator;
		}
		if ( column->visible )
			(*columns)++;
	} while ((list = g_list_next(list)));

	g_list_free (list_start);
}


void aurora_gtk_clist_get_header_index (GtkCList *clist, GtkWidget *button,
                                 gint *column_index, gint *columns)
{
	int i;
	*columns = clist->columns;
	
	for (i=0; i<*columns; i++)
	{
		if (clist->column[i].button == button)
		{
			*column_index = i;
			break;
		}
	}
}


void
aurora_gdk_color_to_cairo (const GdkColor *c, AuroraRGB *cc)
{
	gdouble r, g, b;

	g_return_if_fail (c && cc);

	r = c->red / 65535.0;
	g = c->green / 65535.0;
	b = c->blue / 65535.0;

	cc->r = r;
	cc->g = g;
	cc->b = b;
}

void
aurora_gdk_color_to_rgb (GdkColor *c, double *r, double *g, double *b)
{
	*r = (double)c->red   /  (double)65535;
	*g = (double)c->green /  (double)65535;
	*b = (double)c->blue  /  (double)65535;
}

void
aurora_get_parent_bg (const GtkWidget *widget, AuroraRGB *color)
{
	GtkStateType state_type;
	const GtkWidget *parent;
	GdkColor *gcolor;
	
	if (widget == NULL)
		return;

	
	parent = widget->parent;
	
	while (parent && GTK_WIDGET_NO_WINDOW (parent) 
                       && !( (GTK_IS_NOTEBOOK (parent) && (gtk_notebook_get_show_tabs ((GtkNotebook*)parent) || gtk_notebook_get_show_border((GtkNotebook*)parent)))
                            || GTK_IS_TOOLBAR(parent) 
                            || (GTK_IS_FRAME(parent) && (gtk_frame_get_shadow_type(GTK_FRAME(parent)) != GTK_SHADOW_NONE)) )
         )
		parent = parent->parent;

	if (parent == NULL)
		return;

	if (GTK_IS_MENU_BAR(parent))
		parent = widget;
    
	state_type = GTK_WIDGET_STATE (parent);
	
	gcolor = &parent->style->bg[state_type];
	
	aurora_gdk_color_to_rgb (gcolor, &color->r, &color->g, &color->b);
    
    if (GTK_IS_FRAME (parent)) {
    GtkShadowType shadow = gtk_frame_get_shadow_type(GTK_FRAME(parent));
         if (shadow == (GTK_SHADOW_IN || GTK_SHADOW_ETCHED_IN))
         	aurora_shade (color, color, 0.97);
         else if (shadow == (GTK_SHADOW_OUT || GTK_SHADOW_ETCHED_OUT))
            aurora_shade (color, color, 1.03);
    }
}

==> gtk2-aurora-1.3.tar.bz2: gtk2-aurora-1.3/src/support.c <==
#include "support.h"

GtkTextDirection
aurora_get_direction (GtkWidget *widget)
{
	GtkTextDirection dir;
	
	if (widget)
		dir = gtk_widget_get_direction (widget);
	else
		dir = GTK_TEXT_DIR_LTR;
	
	return dir;
}


/* Widget Type Lookups/Macros
   
   Based on/modified from functions in
   Smooth-Engine.
*/ 
gboolean
aurora_object_is_a (const GObject * object, const gchar * type_name)
{
  gboolean result = FALSE;
 
  if ((object))
    {
      GType tmp = g_type_from_name (type_name);

      if (tmp)
	result = g_type_check_instance_is_a ((GTypeInstance *) object, tmp);
    }
 
  return result;
}

 
gboolean
aurora_is_combo_box_entry (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_COMBO_BOX_ENTRY (widget->parent))
	result = TRUE;
      else
	result = aurora_is_combo_box_entry (widget->parent);
    }
  return result;
}
 
static gboolean
aurora_combo_box_is_using_list (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if (AURORA_IS_COMBO_BOX (widget))
    {
      gboolean *tmp = NULL;
 
      gtk_widget_style_get (widget, "appears-as-list", &result, NULL);

      if (tmp)
	result = *tmp;
    }
 
  return result;
}
 
gboolean
aurora_is_combo_box (GtkWidget * widget, gboolean as_list)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_COMBO_BOX (widget->parent))
        {
          if (as_list)
            result = (aurora_combo_box_is_using_list(widget->parent));
          else
            result = (!aurora_combo_box_is_using_list(widget->parent));
        }
      else
	result = aurora_is_combo_box (widget->parent, as_list);
    }
  return result;
}
 
gboolean
aurora_is_combo (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_COMBO (widget->parent))
	result = TRUE;
      else
	result = aurora_is_combo (widget->parent);
    }
  return result;
}
 
gboolean
aurora_is_in_combo_box (GtkWidget * widget)
{
  return ((aurora_is_combo (widget) || aurora_is_combo_box (widget, TRUE) || aurora_is_combo_box_entry (widget)));
}
 
gboolean
aurora_is_tree_column_header (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_TREE_VIEW (widget->parent))
	result = TRUE;
      else
	result = aurora_is_tree_column_header (widget->parent);
    }
  return result;
}

gboolean
aurora_is_toolbar_item (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent)) {
    if ((AURORA_IS_BONOBO_TOOLBAR (widget->parent))
	|| (AURORA_IS_BONOBO_DOCK_ITEM (widget->parent))
	|| (AURORA_IS_EGG_TOOLBAR (widget->parent))
	|| (AURORA_IS_TOOLBAR (widget->parent))
	|| (AURORA_IS_HANDLE_BOX (widget->parent)))
      result = TRUE;
    else
      result = aurora_is_toolbar_item (widget->parent);
  }
  return result;
}
 
gboolean
aurora_is_panel_widget_item (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget) && (widget->parent))
    {
      if (AURORA_IS_PANEL_WIDGET (widget->parent))
	result = TRUE;
      else
	result = aurora_is_panel_widget_item (widget->parent);
    }
  return result;
}

gboolean 
aurora_is_bonobo_dock_item (GtkWidget * widget)
{
  gboolean result = FALSE;
 
  if ((widget))
    {
      if (AURORA_IS_BONOBO_DOCK_ITEM(widget) || AURORA_IS_BONOBO_DOCK_ITEM (widget->parent))
	result = TRUE;
      else if (AURORA_IS_BOX(widget) || AURORA_IS_BOX(widget->parent))
        {
          GtkContainer *box = AURORA_IS_BOX(widget)?GTK_CONTAINER(widget):GTK_CONTAINER(widget->parent);
          GList *children = NULL, *child = NULL;
 
          children = gtk_container_get_children(box);
              
          for (child = g_list_first(children); child; child = g_list_next(child))
            {
	      if (AURORA_IS_BONOBO_DOCK_ITEM_GRIP(child->data))
	        {
	          result = TRUE;
	          child = NULL;
	        }
            }	            
         
          if (children)   
  	    g_list_free(children);
	}
    }
  return result;
}

static GtkWidget *
aurora_find_combo_box_entry_widget (GtkWidget * widget)
{
  GtkWidget *result = NULL;

  if (widget)
    {
      if (AURORA_IS_COMBO_BOX_ENTRY (widget))
	result = widget;
      else
	result = aurora_find_combo_box_entry_widget (widget->parent);
    }

  return result;
}

static GtkWidget *
aurora_find_combo_box_widget (GtkWidget * widget, gboolean as_list)
{
  GtkWidget *result = NULL;
 
  if (widget)
    {
      if (AURORA_IS_COMBO_BOX (widget))
        {
          if (as_list)
            result = (aurora_combo_box_is_using_list(widget))?widget:NULL;
          else
            result = (!aurora_combo_box_is_using_list(widget))?widget:NULL;
        }
      else
	result = aurora_find_combo_box_widget (widget->parent, as_list);
    }
  return result;
}
 
static GtkWidget *
aurora_find_combo_widget (GtkWidget * widget)
{
  GtkWidget *result = NULL;
 
  if (widget)
    {
      if (AURORA_IS_COMBO (widget))
	result = widget;
      else
	result = aurora_find_combo_widget(widget->parent);
    }
  return result;
}

GtkWidget*
aurora_find_combo_box_widget_parent (GtkWidget * widget)
{
   GtkWidget *result = NULL;
   
   if (!result)
     result = aurora_find_combo_widget(widget);
  
   if (!result)
     result = aurora_find_combo_box_widget(widget, TRUE);

   if (!result)
     result = aurora_find_combo_box_entry_widget(widget);

  return result;
}


gboolean
aurora_widget_is_ltr (GtkWidget *widget)
{
	GtkTextDirection dir = GTK_TEXT_DIR_NONE;
	
	if (AURORA_IS_WIDGET (widget))
		dir = gtk_widget_get_direction (widget);

	if (dir == GTK_TEXT_DIR_NONE)
		dir = gtk_widget_get_default_direction ();

	if (dir == GTK_TEXT_DIR_RTL)
		return FALSE;
	else
		return TRUE;
}




/***********************************************
 * ge_hsb_from_color -
 *  
 *   Get HSB values from RGB values.
 *
 *   Modified from Smooth but originated in GTK+
 ***********************************************/
static void
aurora_hsb_from_color (const AuroraRGB *color, 
                        gdouble *hue, 
                        gdouble *saturation,
                        gdouble *brightness) 
{
	gdouble min, max, delta;
	gdouble red, green, blue;

	red = color->r;
	green = color->g;
	blue = color->b;
  
	if (red > green)
	{
		max = MAX(red, blue);
		min = MIN(green, blue);
	}
	else
	{
		max = MAX(green, blue);
		min = MIN(red, blue);
	}
  
	*brightness = (max + min) / 2;
 	
	if (fabs(max - min) < 0.0001)
	{
		*hue = 0;
		*saturation = 0;
	}	
	else
	{
		if (*brightness <= 0.5)
			*saturation = (max - min) / (max + min);
		else
			*saturation = (max - min) / (2 - max - min);
       
		delta = max -min;
 
		if (red == max)
			*hue = (green - blue) / delta;
		else if (green == max)
			*hue = 2 + (blue - red) / delta;
		else if (blue == max)
			*hue = 4 + (red - green) / delta;
 
		*hue *= 60;
		if (*hue < 0.0)
			*hue += 360;
	}
}
 
/***********************************************
 * ge_color_from_hsb -
 *  
 *   Get RGB values from HSB values.
 *
 *   Modified from Smooth but originated in GTK+
 ***********************************************/
#define MODULA(number, divisor) (((gint)number % divisor) + (number - (gint)number))
static void
aurora_color_from_hsb (gdouble hue, 
                        gdouble saturation,
                        gdouble brightness, 
                        AuroraRGB *color)
{
	gint i;
	gdouble hue_shift[3], color_shift[3];
	gdouble m1, m2, m3;

	if (!color) return;
  	  
	if (brightness <= 0.5)
		m2 = brightness * (1 + saturation);
	else
		m2 = brightness + saturation - brightness * saturation;
 
	m1 = 2 * brightness - m2;
 
	hue_shift[0] = hue + 120;
	hue_shift[1] = hue;
	hue_shift[2] = hue - 120;
 
	color_shift[0] = color_shift[1] = color_shift[2] = brightness;	
 
	i = (saturation == 0)?3:0;
 
	for (; i < 3; i++)
	{
		m3 = hue_shift[i];
 
		if (m3 > 360)
			m3 = MODULA(m3, 360);
		else if (m3 < 0)
			m3 = 360 - MODULA(ABS(m3), 360);
 
		if (m3 < 60)
			color_shift[i] = m1 + (m2 - m1) * m3 / 60;
		else if (m3 < 180)
			color_shift[i] = m2;
		else if (m3 < 240)
			color_shift[i] = m1 + (m2 - m1) * (240 - m3) / 60;
		else
			color_shift[i] = m1;
	}	
 
	color->r = color_shift[0];
	color->g = color_shift[1];
	color->b = color_shift[2];	
}

void
aurora_shade (const AuroraRGB *base, AuroraRGB *composite, double shade_ratio)
{
	gdouble hue = 0;
	gdouble saturation = 0;
	gdouble brightness = 0;
 
	g_return_if_fail (base && composite);

	aurora_hsb_from_color (base, &hue, &saturation, &brightness);
 
	brightness = MIN(brightness*shade_ratio, 1.0);
	brightness = MAX(brightness, 0.0);
	
	aurora_color_from_hsb (hue, saturation, brightness, composite);
}

void
aurora_shade_shift (const AuroraRGB *base, AuroraRGB *composite, double shade_ratio)
{
	gdouble hue = 0;
	gdouble saturation = 0;
	gdouble brightness = 0;
 
	g_return_if_fail (base && composite);

	aurora_hsb_from_color (base, &hue, &saturation, &brightness);
 
	brightness = MIN(brightness*shade_ratio, 1.0);
	brightness = MAX(brightness, 0.0);
	hue += shade_ratio >= 1.0 ? -2.75*shade_ratio : -12*shade_ratio + 12;
	
	aurora_color_from_hsb (hue, saturation, brightness, composite);
}

void
aurora_hue_shift (const AuroraRGB *base, AuroraRGB *composite, double shift_amount)
{
	gdouble hue = 0;
	gdouble saturation = 0;
	gdouble brightness = 0;
 
	g_return_if_fail (base && composite);

	aurora_hsb_from_color (base, &hue, &saturation, &brightness);
 
	hue += shift_amount;
	
	aurora_color_from_hsb (hue, saturation, brightness, composite);
}

void
aurora_match_lightness (const AuroraRGB * a, AuroraRGB * b)
{
		
	gdouble hue = 0;
	gdouble saturation = 0;
	gdouble brightness = 0;
 
	aurora_hsb_from_color (b, &hue, &saturation, &brightness);
	aurora_color_from_hsb (hue, saturation, aurora_get_lightness(a), b);
		
}

void
aurora_set_lightness (AuroraRGB * a, double new_lightness)
{
		
	gdouble hue = 0;
	gdouble saturation = 0;
	gdouble brightness = 0;
 
	aurora_hsb_from_color (a, &hue, &saturation, &brightness);
	aurora_color_from_hsb (hue, saturation, new_lightness, a);
		
}

void
aurora_tweak_saturation (const AuroraRGB * a, AuroraRGB * b)
{
		
	gdouble hue = 0;
	gdouble hue2 = 0;
	
	gdouble saturation = 0;
	gdouble saturation2 = 0;
	
	gdouble brightness = 0;
	gdouble brightness2 = 0;
	
	aurora_hsb_from_color (b, &hue, &saturation, &brightness);
 	aurora_hsb_from_color (a, &hue2, &saturation2, &brightness2);

	
	aurora_color_from_hsb (hue, (saturation+saturation2)/2.0, brightness, b);
		
}


gdouble
aurora_get_lightness (const AuroraRGB * a)
{

	gdouble min, max;
 
	if (a->r > a->g)
	{
		max = MAX(a->r, a->b);
		min = MIN(a->g, a->b);
	}
	else
	{
		max = MAX(a->g, a->b);
		min = MIN(a->r, a->b);
	}
  	
	return (max + min) / 2.0;
}


void
aurora_mix_color (const AuroraRGB * a, const AuroraRGB * b, gdouble mix_factor, AuroraRGB * c)
{
	g_return_if_fail (a && b && c);

	c->r = a->r * (1-mix_factor) + b->r * mix_factor;
	c->g = a->g * (1-mix_factor) + b->g * mix_factor;
	c->b = a->b * (1-mix_factor) + b->b * mix_factor;
}


void aurora_gtk_treeview_get_header_index (GtkTreeView *tv, GtkWidget *header,
                                    gint *column_index, gint *columns,
                                    gboolean *resizable, gboolean *sorted)
{
	GList *list, *list_start;
	*column_index = *columns = 0;
	list_start = list = gtk_tree_view_get_columns (tv);

	do
	{
		GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN(list->data);
		if ( column->button == header )
		{
			*column_index = *columns;
			*resizable = column->resizable;
			*sorted = column->show_sort_indicator;
		}
		if ( column->visible )
			(*columns)++;
	} while ((list = g_list_next(list)));

	g_list_free (list_start);
}


void aurora_gtk_clist_get_header_index (GtkCList *clist, GtkWidget *button,
                                 gint *column_index, gint *columns)
{
	int i;
	*columns = clist->columns;
	
	for (i=0; i<*columns; i++)
	{
		if (clist->column[i].button == button)
		{
			*column_index = i;
			break;
		}
	}
}


void
aurora_gdk_color_to_cairo (const GdkColor *c, AuroraRGB *cc)
{
	gdouble r, g, b;

	g_return_if_fail (c && cc);

	r = c->red / 65535.0;
	g = c->green / 65535.0;
	b = c->blue / 65535.0;

	cc->r = r;
	cc->g = g;
	cc->b = b;
}

void
aurora_gdk_color_to_rgb (GdkColor *c, double *r, double *g, double *b)
{
	*r = (double)c->red   /  (double)65535;
	*g = (double)c->green /  (double)65535;
	*b = (double)c->blue  /  (double)65535;
}

void
aurora_get_parent_bg (const GtkWidget *widget, AuroraRGB *color)
{
	GtkStateType state_type;
	const GtkWidget *parent;
	GdkColor *gcolor;
	
	if (widget == NULL)
		return;

	
	parent = widget->parent;
	
	while (parent && GTK_WIDGET_NO_WINDOW (parent) 
                       && !( (GTK_IS_NOTEBOOK (parent) && (gtk_notebook_get_show_tabs ((GtkNotebook*)parent) || gtk_notebook_get_show_border((GtkNotebook*)parent)))
                            || GTK_IS_TOOLBAR(parent) 
                            || (GTK_IS_FRAME(parent) && (gtk_frame_get_shadow_type(GTK_FRAME(parent)) != GTK_SHADOW_NONE)) )
		)
		parent = parent->parent;

	if (parent == NULL)
		return;

	if (GTK_IS_MENU_BAR(parent))
		parent = widget;
    
	state_type = GTK_WIDGET_STATE (parent);
	
	gcolor = &parent->style->bg[state_type];
	
	aurora_gdk_color_to_rgb (gcolor, &color->r, &color->g, &color->b);
    
    if (GTK_IS_FRAME (parent)) {
    GtkShadowType shadow = gtk_frame_get_shadow_type(GTK_FRAME(parent));
         if (shadow == (GTK_SHADOW_IN || GTK_SHADOW_ETCHED_IN))
         	aurora_shade (color, color, 0.97);
         else if (shadow == (GTK_SHADOW_OUT || GTK_SHADOW_ETCHED_OUT))
            aurora_shade (color, color, 1.03);
    }
}
